@@ -2,7 +2,7 @@ use std::thread;
 
 use crate::{
     backend::{Backend, BackendResult, LogEntry},
-    mode::{ModeContext, ModeKind, ModeResponse, ModeStatus, Output, SelectMenu},
+    mode::{Filter, ModeContext, ModeKind, ModeResponse, ModeStatus, Output, SelectMenu},
     platform::Key,
     ui::{Color, Drawer, SelectEntryDraw, RESERVED_LINES_COUNT},
 };
@@ -40,7 +40,8 @@ impl SelectEntryDraw for LogEntry {
             }
         }
 
-        let author = match self.author.char_indices().nth(12) {
+        const MAX_AUTHOR_CHAR_COUNT: usize = 12;
+        let author = match self.author.char_indices().nth(MAX_AUTHOR_CHAR_COUNT) {
             Some((i, _)) => &self.author[..i],
             None => &self.author,
         };
@@ -127,6 +128,7 @@ pub struct Mode {
     entries: Vec<LogEntry>,
     output: Output,
     select: SelectMenu,
+    filter: Filter,
     show_full_hovered_message: bool,
 }
 impl Mode {
@@ -137,73 +139,90 @@ impl Mode {
         self.state = State::Waiting(WaitOperation::Refresh);
 
         self.output.set(String::new());
+        self.filter.filter(self.entries.iter());
+        self.select
+            .saturate_cursor(self.filter.visible_indices().len());
         self.show_full_hovered_message = false;
 
         request(ctx, |_| Ok(()));
     }
 
     pub fn on_key(&mut self, ctx: &ModeContext, key: Key) -> ModeStatus {
+        let pending_input = self.filter.has_focus();
         let available_height = (ctx.viewport_size.1 as usize).saturating_sub(RESERVED_LINES_COUNT);
 
-        self.select
-            .on_key(self.entries.len(), available_height, key);
+        if self.filter.has_focus() {
+            self.filter.on_key(key);
 
-        if matches!(self.state, State::Idle) && self.select.cursor() + 1 == self.entries.len() {
-            self.state = State::Waiting(WaitOperation::Refresh);
-            let start = self.entries.len();
-            let ctx = ctx.clone();
-            thread::spawn(move || {
-                let result = ctx.backend.log(start, available_height);
-                ctx.event_sender
-                    .send_response(ModeResponse::Log(Response::Refresh(result)));
-            });
-        }
+            self.filter.filter(self.entries.iter());
+            self.select
+                .saturate_cursor(self.filter.visible_indices().len());
+        } else {
+            self.select
+                .on_key(self.filter.visible_indices().len(), available_height, key);
 
-        if let Key::Char('d') = key {
-            let index = self.select.cursor();
-            if let Some(entry) = self.entries.get(index) {
-                ctx.event_sender
-                    .send_mode_change(ModeKind::RevisionDetails(entry.hash.clone()));
+            let current_entry_index = self.filter.get_visible_index(self.select.cursor);
+            if matches!(self.state, State::Idle)
+                && current_entry_index
+                    .map(|i| i + 1 == self.entries.len())
+                    .unwrap_or(false)
+            {
+                self.state = State::Waiting(WaitOperation::Refresh);
+                let start = self.entries.len();
+                let ctx = ctx.clone();
+                thread::spawn(move || {
+                    let result = ctx.backend.log(start, available_height);
+                    ctx.event_sender
+                        .send_response(ModeResponse::Log(Response::Refresh(result)));
+                });
             }
-        } else if let Key::Tab = key {
-            self.show_full_hovered_message = !self.show_full_hovered_message;
-        } else if let State::Idle = self.state {
-            match key {
-                Key::Char('g') => {
-                    let index = self.select.cursor();
-                    if let Some(entry) = self.entries.get(index) {
-                        self.state = State::Waiting(WaitOperation::Checkout);
-                        let revision = entry.hash.clone();
-                        request(ctx, move |b| b.checkout(&revision));
+
+            if let Key::Char('d') = key {
+                if let Some(current_entry_index) = current_entry_index {
+                    let entry = &self.entries[current_entry_index];
+                    ctx.event_sender
+                        .send_mode_change(ModeKind::RevisionDetails(entry.hash.clone()));
+                }
+            } else if let Key::Tab = key {
+                self.show_full_hovered_message = !self.show_full_hovered_message;
+            } else if let Key::Ctrl('f') = key {
+                self.filter.enter();
+            } else if let State::Idle = self.state {
+                match key {
+                    Key::Char('g') => {
+                        if let Some(current_entry_index) = current_entry_index {
+                            let entry = &self.entries[current_entry_index];
+                            self.state = State::Waiting(WaitOperation::Checkout);
+                            let revision = entry.hash.clone();
+                            request(ctx, move |b| b.checkout(&revision));
+                        }
                     }
-                }
-                Key::Char('m') => {
-                    let index = self.select.cursor();
-                    if let Some(entry) = self.entries.get(index) {
-                        self.state = State::Waiting(WaitOperation::Merge);
-                        let revision = entry.hash.clone();
-                        request(ctx, move |b| b.merge(&revision));
+                    Key::Char('m') => {
+                        if let Some(current_entry_index) = current_entry_index {
+                            let entry = &self.entries[current_entry_index];
+                            self.state = State::Waiting(WaitOperation::Merge);
+                            let revision = entry.hash.clone();
+                            request(ctx, move |b| b.merge(&revision));
+                        }
                     }
+                    Key::Char('f') => {
+                        self.state = State::Waiting(WaitOperation::Fetch);
+                        request(ctx, Backend::fetch);
+                    }
+                    Key::Char('p') => {
+                        self.state = State::Waiting(WaitOperation::Pull);
+                        request(ctx, Backend::pull);
+                    }
+                    Key::Char('P') => {
+                        self.state = State::Waiting(WaitOperation::Push);
+                        request(ctx, Backend::push);
+                    }
+                    _ => (),
                 }
-                Key::Char('f') => {
-                    self.state = State::Waiting(WaitOperation::Fetch);
-                    request(ctx, Backend::fetch);
-                }
-                Key::Char('p') => {
-                    self.state = State::Waiting(WaitOperation::Pull);
-                    request(ctx, Backend::pull);
-                }
-                Key::Char('P') => {
-                    self.state = State::Waiting(WaitOperation::Push);
-                    request(ctx, Backend::push);
-                }
-                _ => (),
             }
         }
 
-        ModeStatus {
-            pending_input: false,
-        }
+        ModeStatus { pending_input }
     }
 
     pub fn on_response(&mut self, response: Response) {
@@ -227,7 +246,9 @@ impl Mode {
                     }
                 }
 
-                self.select.saturate_cursor(self.entries.len());
+                self.filter.filter(self.entries.iter());
+                self.select
+                    .saturate_cursor(self.filter.visible_indices().len());
             }
         }
     }
@@ -248,18 +269,23 @@ impl Mode {
             State::Waiting(WaitOperation::Pull) => "pull",
             State::Waiting(WaitOperation::Push) => "push",
         };
+
         let left_help = "[g]checkout [d]details [f]fetch [p]pull [P]push";
-        let right_help = "[arrows]move";
+        let right_help = "[tab]full message [arrows]move [ctrl+f]filter";
         (name, left_help, right_help)
     }
 
     pub fn draw(&self, drawer: &mut Drawer) {
+        let filter_line_count = drawer.filter(&self.filter);
         if self.output.text().is_empty() {
             drawer.select_menu(
                 &self.select,
-                0,
+                filter_line_count,
                 self.show_full_hovered_message,
-                self.entries.iter(),
+                self.filter
+                    .visible_indices()
+                    .iter()
+                    .map(|&i| &self.entries[i]),
             );
         } else {
             drawer.output(&self.output);
